@@ -119,6 +119,7 @@ typedef enum cl_error_t {
     CL_ESTATE,
 
     CL_VERIFIED, /* The binary has been deemed trusted */
+    CL_ERROR,    /* Unspecified / generic error */
 
     /* no error codes below this line please */
     CL_ELAST_ERROR
@@ -139,7 +140,7 @@ typedef enum cl_error_t {
 #define CL_DB_OFFICIAL_ONLY     0x1000
 #define CL_DB_BYTECODE          0x2000
 #define CL_DB_SIGNED            0x4000  /* internal */
-#define CL_DB_BYTECODE_UNSIGNED 0x8000
+#define CL_DB_BYTECODE_UNSIGNED 0x8000  /* Caution: You should never run bytecode signatures from untrusted sources. Doing so may result in arbitrary code execution. */
 #define CL_DB_UNSIGNED          0x10000 /* internal */
 #define CL_DB_BYTECODE_STATS    0x20000
 #define CL_DB_ENHANCED          0x40000
@@ -179,17 +180,19 @@ struct cl_scan_options {
 #define CL_SCAN_PARSE_PE                            0x200
 
 /* heuristic alerting options */
-#define CL_SCAN_HEURISTIC_BROKEN                    0x2   /* alert on broken PE and broken ELF files */
-#define CL_SCAN_HEURISTIC_EXCEEDS_MAX               0x4   /* alert when files exceed scan limits (filesize, max scansize, or max recursion depth) */
-#define CL_SCAN_HEURISTIC_PHISHING_SSL_MISMATCH     0x8   /* alert on SSL mismatches */
-#define CL_SCAN_HEURISTIC_PHISHING_CLOAK            0x10  /* alert on cloaked URLs in emails */
-#define CL_SCAN_HEURISTIC_MACROS                    0x20  /* alert on OLE2 files containing macros */
-#define CL_SCAN_HEURISTIC_ENCRYPTED_ARCHIVE         0x40  /* alert if archive is encrypted (rar, zip, etc) */
-#define CL_SCAN_HEURISTIC_ENCRYPTED_DOC             0x80  /* alert if a document is encrypted (pdf, docx, etc) */
-#define CL_SCAN_HEURISTIC_PARTITION_INTXN           0x100 /* alert if partition table size doesn't make sense */
-#define CL_SCAN_HEURISTIC_STRUCTURED                0x200 /* data loss prevention options, i.e. alert when detecting personal information */
-#define CL_SCAN_HEURISTIC_STRUCTURED_SSN_NORMAL     0x400 /* alert when detecting social security numbers */
-#define CL_SCAN_HEURISTIC_STRUCTURED_SSN_STRIPPED   0x800 /* alert when detecting stripped social security numbers */
+#define CL_SCAN_HEURISTIC_BROKEN                    0x2    /* alert on broken PE and broken ELF files */
+#define CL_SCAN_HEURISTIC_EXCEEDS_MAX               0x4    /* alert when files exceed scan limits (filesize, max scansize, or max recursion depth) */
+#define CL_SCAN_HEURISTIC_PHISHING_SSL_MISMATCH     0x8    /* alert on SSL mismatches */
+#define CL_SCAN_HEURISTIC_PHISHING_CLOAK            0x10   /* alert on cloaked URLs in emails */
+#define CL_SCAN_HEURISTIC_MACROS                    0x20   /* alert on OLE2 files containing macros */
+#define CL_SCAN_HEURISTIC_ENCRYPTED_ARCHIVE         0x40   /* alert if archive is encrypted (rar, zip, etc) */
+#define CL_SCAN_HEURISTIC_ENCRYPTED_DOC             0x80   /* alert if a document is encrypted (pdf, docx, etc) */
+#define CL_SCAN_HEURISTIC_PARTITION_INTXN           0x100  /* alert if partition table size doesn't make sense */
+#define CL_SCAN_HEURISTIC_STRUCTURED                0x200  /* data loss prevention options, i.e. alert when detecting personal information */
+#define CL_SCAN_HEURISTIC_STRUCTURED_SSN_NORMAL     0x400  /* alert when detecting social security numbers */
+#define CL_SCAN_HEURISTIC_STRUCTURED_SSN_STRIPPED   0x800  /* alert when detecting stripped social security numbers */
+
+#define CL_SCAN_HEURISTIC_STRUCTURED_CC             0x1000 /* alert when detecting credit card numbers */
 
 /* mail scanning options */
 #define CL_SCAN_MAIL_PARTIAL_MESSAGE                0x1
@@ -260,7 +263,7 @@ void cl_cleanup_crypto(void);
  * @param initoptions   Unused.
  * @return cl_error_t   CL_SUCCESS if everything initalized correctly.
  */
-extern int cl_init(unsigned int initoptions);
+extern cl_error_t cl_init(unsigned int initoptions);
 
 /**
  * @brief Allocate a new scanning engine and initialize default settings.
@@ -337,6 +340,8 @@ typedef struct cli_stats_sections {
 /**
  * @brief Set a numerical engine option.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine            An initialized scan engine.
  * @param cl_engine_field   A CL_ENGINE option.
  * @param num               The new engine option value.
@@ -344,7 +349,7 @@ typedef struct cli_stats_sections {
  * @return cl_error_t       CL_EARG if the field number was incorrect.
  * @return cl_error_t       CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_set_num(struct cl_engine *engine, enum cl_engine_field field, long long num);
+extern cl_error_t cl_engine_set_num(struct cl_engine *engine, enum cl_engine_field field, long long num);
 
 /**
  * @brief Get a numerical engine option.
@@ -359,6 +364,11 @@ extern long long cl_engine_get_num(const struct cl_engine *engine, enum cl_engin
 /**
  * @brief Set a string engine option.
  *
+ * If the string option has already been set, the existing string will be free'd
+ * and the new string will replace it.
+ *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine            An initialized scan engine.
  * @param cl_engine_field   A CL_ENGINE option.
  * @param str               The new engine option value.
@@ -367,7 +377,7 @@ extern long long cl_engine_get_num(const struct cl_engine *engine, enum cl_engin
  * @return cl_error_t       CL_EMEM if a memory allocation error occurred.
  * @return cl_error_t       CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_set_str(struct cl_engine *engine, enum cl_engine_field field, const char *str);
+extern cl_error_t cl_engine_set_str(struct cl_engine *engine, enum cl_engine_field field, const char *str);
 
 /**
  * @brief Get a string engine option.
@@ -392,12 +402,14 @@ extern struct cl_settings *cl_engine_settings_copy(const struct cl_engine *engin
 /**
  * @brief Apply settings from a settings structure to a scan engine.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine        A scan engine.
  * @param settings      The settings.
  * @return cl_error_t   CL_SUCCESS if successful.
  * @return cl_error_t   CL_EMEM if a memory allocation error occurred.
  */
-extern int cl_engine_settings_apply(struct cl_engine *engine, const struct cl_settings *settings);
+extern cl_error_t cl_engine_settings_apply(struct cl_engine *engine, const struct cl_settings *settings);
 
 /**
  * @brief Free a settings struct pointer.
@@ -406,7 +418,7 @@ extern int cl_engine_settings_apply(struct cl_engine *engine, const struct cl_se
  * @return cl_error_t   CL_SUCCESS if successful.
  * @return cl_error_t   CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_settings_free(struct cl_settings *settings);
+extern cl_error_t cl_engine_settings_free(struct cl_settings *settings);
 
 /**
  * @brief Prepare the scanning engine.
@@ -418,7 +430,7 @@ extern int cl_engine_settings_free(struct cl_settings *settings);
  * @return cl_error_t   CL_SUCCESS if successful.
  * @return cl_error_t   CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_compile(struct cl_engine *engine);
+extern cl_error_t cl_engine_compile(struct cl_engine *engine);
 
 /**
  * @brief Add a reference count to the engine.
@@ -432,7 +444,7 @@ extern int cl_engine_compile(struct cl_engine *engine);
  * @return cl_error_t   CL_SUCCESS if successful.
  * @return cl_error_t   CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_addref(struct cl_engine *engine);
+extern cl_error_t cl_engine_addref(struct cl_engine *engine);
 
 /**
  * @brief Free an engine.
@@ -444,7 +456,7 @@ extern int cl_engine_addref(struct cl_engine *engine);
  * @return cl_error_t   CL_SUCCESS if successful.
  * @return cl_error_t   CL_ENULLARG null arguments were provided.
  */
-extern int cl_engine_free(struct cl_engine *engine);
+extern cl_error_t cl_engine_free(struct cl_engine *engine);
 
 /* ----------------------------------------------------------------------------
  * Callback function type definitions.
@@ -468,6 +480,8 @@ typedef cl_error_t (*clcb_pre_cache)(int fd, const char *type, void *context);
 /**
  * @brief Set a custom pre-cache callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -492,6 +506,8 @@ typedef cl_error_t (*clcb_pre_scan)(int fd, const char *type, void *context);
 /**
  * @brief Set a custom pre-scan callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -517,6 +533,8 @@ typedef cl_error_t (*clcb_post_scan)(int fd, int result, const char *virname, vo
 /**
  * @brief Set a custom post-scan callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -540,6 +558,8 @@ typedef void (*clcb_virus_found)(int fd, const char *virname, void *context);
 /**
  * @brief Set a custom virus-found callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -565,6 +585,8 @@ typedef int (*clcb_sigload)(const char *type, const char *name, unsigned int cus
 /**
  * @brief Set a custom signature-load callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  * @param context   Opaque application provided data.
@@ -625,6 +647,8 @@ typedef void (*clcb_hash)(int fd, unsigned long long size, const unsigned char *
 /**
  * @brief Set a custom hash stats callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -654,6 +678,8 @@ typedef cl_error_t (*clcb_meta)(const char *container_type, unsigned long fsize_
 /**
  * @brief Set a custom archive metadata matching callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -673,6 +699,8 @@ typedef int (*clcb_file_props)(const char *j_propstr, int rc, void *cbdata);
 /**
  * @brief Set a custom file properties callback function.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -694,6 +722,8 @@ extern void cl_engine_set_clcb_file_props(struct cl_engine *engine, clcb_file_pr
  * The data must persist at least until `clcb_stats_submit()` is called, or
  * `clcb_stats_flush()` is called (optional).
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine The scanning engine.
  * @param cbdata The statistics data. Probably a pointer to a malloc'd struct.
  */
@@ -712,6 +742,8 @@ typedef void (*clcb_stats_add_sample)(const char *virname, const unsigned char *
 /**
  * @brief Set a custom callback function to add sample metadata to a statistics report.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -729,6 +761,8 @@ typedef void (*clcb_stats_remove_sample)(const char *virname, const unsigned cha
 /**
  * @brief Set a custom callback function to remove sample metadata from a statistics report.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -763,6 +797,8 @@ typedef void (*clcb_stats_submit)(struct cl_engine *engine, void *cbdata);
 /**
  * @brief Set a custom callback function to submit the statistics report.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -778,6 +814,8 @@ typedef void (*clcb_stats_flush)(struct cl_engine *engine, void *cbdata);
 /**
  * @brief Set a custom callback function to flush/free the statistics report data.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -792,6 +830,8 @@ typedef size_t (*clcb_stats_get_num)(void *cbdata);
 /**
  * @brief Set a custom callback function to get the number of samples listed in the statistics report.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -806,6 +846,8 @@ typedef size_t (*clcb_stats_get_size)(void *cbdata);
 /**
  * @brief Set a custom callback function to get the size of memory used to store the statistics report.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -820,6 +862,8 @@ typedef char *(*clcb_stats_get_hostid)(void *cbdata);
 /**
  * @brief Set a custom callback function to get the machine's unique host ID.
  *
+ * Caution: changing options for an engine that is in-use is not thread-safe!
+ *
  * @param engine    The initialized scanning engine.
  * @param callback  The callback function pointer.
  */
@@ -847,7 +891,7 @@ extern void cl_engine_stats_enable(struct cl_engine *engine);
  * @param scanoptions       Scanning options.
  * @return cl_error_t       CL_CLEAN, CL_VIRUS, or an error code if an error occured during the scan.
  */
-extern int cl_scandesc(int desc, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions);
+extern cl_error_t cl_scandesc(int desc, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions);
 
 /**
  * @brief Scan a file, given a file descriptor.
@@ -863,7 +907,7 @@ extern int cl_scandesc(int desc, const char *filename, const char **virname, uns
  * @param[in/out] context   An opaque context structure allowing the caller to record details about the sample being scanned.
  * @return cl_error_t       CL_CLEAN, CL_VIRUS, or an error code if an error occured during the scan.
  */
-extern int cl_scandesc_callback(int desc, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
+extern cl_error_t cl_scandesc_callback(int desc, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
 
 /**
  * @brief Scan a file, given a filename.
@@ -875,7 +919,7 @@ extern int cl_scandesc_callback(int desc, const char *filename, const char **vir
  * @param scanoptions       Scanning options.
  * @return cl_error_t       CL_CLEAN, CL_VIRUS, or an error code if an error occured during the scan.
  */
-extern int cl_scanfile(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions);
+extern cl_error_t cl_scanfile(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions);
 
 /**
  * @brief Scan a file, given a filename.
@@ -890,12 +934,12 @@ extern int cl_scanfile(const char *filename, const char **virname, unsigned long
  * @param[in/out] context   An opaque context structure allowing the caller to record details about the sample being scanned.
  * @return cl_error_t       CL_CLEAN, CL_VIRUS, or an error code if an error occured during the scan.
  */
-extern int cl_scanfile_callback(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
+extern cl_error_t cl_scanfile_callback(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
 
 /* ----------------------------------------------------------------------------
  * Database handling.
  */
-extern int cl_load(const char *path, struct cl_engine *engine, unsigned int *signo, unsigned int dboptions);
+extern cl_error_t cl_load(const char *path, struct cl_engine *engine, unsigned int *signo, unsigned int dboptions);
 extern const char *cl_retdbdir(void);
 
 /* ----------------------------------------------------------------------------
@@ -943,7 +987,7 @@ extern struct cl_cvd *cl_cvdparse(const char *head);
  * @param file          Filepath of CVD file.
  * @return cl_error_t   CL_SUCCESS if success, else a CL_E* error code.
  */
-extern int cl_cvdverify(const char *file);
+extern cl_error_t cl_cvdverify(const char *file);
 
 /**
  * @brief Free a CVD header struct.
@@ -973,7 +1017,7 @@ struct cl_stat {
  * @param[out] dbstat   dbstat handle.
  * @return cl_error_t   CL_SUCCESS if successfully initialized.
  */
-extern int cl_statinidir(const char *dirname, struct cl_stat *dbstat);
+extern cl_error_t cl_statinidir(const char *dirname, struct cl_stat *dbstat);
 
 /**
  * @brief Check the database directory for changes.
@@ -991,7 +1035,7 @@ extern int cl_statchkdir(const struct cl_stat *dbstat);
  * @return cl_error_t   CL_SUCCESS
  * @return cl_error_t   CL_ENULLARG
  */
-extern int cl_statfree(struct cl_stat *dbstat);
+extern cl_error_t cl_statfree(struct cl_stat *dbstat);
 
 /**
  * @brief Count the number of signatures in a database file or directory.
@@ -1001,7 +1045,7 @@ extern int cl_statfree(struct cl_stat *dbstat);
  * @param[out] sigs     The number of sigs.
  * @return cl_error_t   CL_SUCCESS if success, else a CL_E* error type.
  */
-extern int cl_countsigs(const char *path, unsigned int countoptions, unsigned int *sigs);
+extern cl_error_t cl_countsigs(const char *path, unsigned int countoptions, unsigned int *sigs);
 
 /* ----------------------------------------------------------------------------
  * Software versions.
@@ -1119,7 +1163,7 @@ extern void cl_fmap_close(cl_fmap_t *);
  *                      signature matched. Another CL_E* error code if an
  *                      error occured.
  */
-extern int cl_scanmap_callback(cl_fmap_t *map, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
+extern cl_error_t cl_scanmap_callback(cl_fmap_t *map, const char *filename, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, struct cl_scan_options *scanoptions, void *context);
 
 /* ----------------------------------------------------------------------------
  * Crypto/hashing functions
